@@ -5,7 +5,7 @@ from pandas.api.types import is_string_dtype, is_numeric_dtype, is_datetime64_an
 from ipykernel.comm import Comm # type: ignore
 import numpy as np
 # from json import loads
-from typing import Dict, Callable, Optional, List, Tuple, cast
+from typing import Dict, Callable, Optional, List, Tuple, Type, cast
 import json
 from pyperclip import copy
 import ast
@@ -23,9 +23,9 @@ from midas.state_types import DFName
 from midas.midas_algebra.dataframe import MidasDataFrame, RelationalOp, DFInfo, VisualizedDFInfo, get_midas_code
 from midas.midas_algebra.selection import NumericRangeSelection, SetSelection, ColumnRef, EmptySelection
 from .util.errors import InternalLogicalError, MockComm, debug_log, NotAllCaseHandledError
-from .util.utils import sanitize_string_for_var_name, get_basic_group_vis
+from .util.utils import sanitize_string_for_var_name
 from .vis_types import EncodingSpec, FilterLabelOptions
-from .util.data_processing import dataframe_to_dict, get_numeric_distribution_code, get_datetime_distribution_code
+from .util.data_processing import dataframe_to_dict, get_numeric_distribution_code, get_datetime_distribution_code, get_basic_group_vis
 
 def logged(remove_on_chart_removal: bool):
     def wrapper_factory(f):
@@ -58,25 +58,26 @@ class UiComm(object):
     midas_instance_name: str
     create_df_from_ops:  Callable[[RelationalOp], MidasDataFrame]
 
-    def __init__(self, is_in_ipynb: bool, midas_instance_name: str,
-      get_df_fun: Callable[[DFName], Optional[DFInfo]],
-      remove_df_fun: Callable[[DFName], None],
-      create_df_from_ops: Callable[[RelationalOp], MidasDataFrame],
-      add_selection: Callable[[List[SelectionValue]], List[SelectionValue]],
-      get_filtered_code: Callable[[str], str],
-      log_entry: Callable[[str, Optional[str]], None]):
+    def __init__(self,
+        is_in_ipynb: bool,
+        midas_instance_name: str,
+        logger_id: str,
+        get_df_fun: Callable[[DFName], Optional[DFInfo]],
+        remove_df_fun: Callable[[DFName], None],
+        create_df_from_ops: Callable[[RelationalOp], MidasDataFrame],
+        add_selection: Callable[[List[SelectionValue]], List[SelectionValue]],
+        get_filtered_code: Callable[[str], str]):
 
         # functions passed at creation time
         self.is_in_ipynb = is_in_ipynb
         self.midas_instance_name = midas_instance_name
-        self.set_comm(midas_instance_name)
-        self.register_recovery_comm(midas_instance_name)
+        self.set_comm(midas_instance_name, logger_id)
+        self.register_recovery_comm(midas_instance_name, logger_id)
         self.get_df = get_df_fun
         self.remove_df_fun = remove_df_fun
         self.create_df_from_ops = create_df_from_ops
         self.add_selection = add_selection
         self.get_filtered_code = get_filtered_code
-        self.log_entry = log_entry
 
         # internal state
         self.next_id = 0
@@ -87,7 +88,6 @@ class UiComm(object):
         self.tmp_log = []
 
     def log(self, function, args, kwargs, associated_df_name: str):
-        # , fun_name: str
         self.logged_comms.append((
             function,           # 0
             args,               # 1
@@ -108,19 +108,15 @@ class UiComm(object):
     def handle_msg(self, data_raw):
         data = data_raw["content"]["data"]
         self.tmp_log.append(data)
-        debug_log(f"got message {data}")
         if "command" in data:
             command = data["command"]
             if command == "cell-ran":
                 if "code" in data:
                     code = data["code"]
-                    # self.process_code(code)
-                    self.log_entry("code_execution", json.dumps(code))
                 return
             elif command == "markdown-cell-rendered":
                 if "code" in data:
                     code = data["code"]
-                    self.log_entry("markdown-rendered", json.dumps(code))
                 return
             elif command == "get-visualization-code-clipboard":
                 df_name = data["df_name"]
@@ -133,43 +129,38 @@ class UiComm(object):
                 df_name = DFName(data["df_name"])
                 self.tmp = f"{column}_{df_name}"
                 code, execute, err_message = self.create_distribution_query(column, df_name)
-                self.log_entry("column_click", df_name)
                 if code:
                     if execute:
                         # creating new line so that the horizontal scroll wouldn't occlude the code
-                        self.create_cell(code + "\n", "query", True)
+                        self.create_cell(code, "query", True)
                     else:
                         self.send_column_click_error_msg(
                             f'We are not able to create a chart for {df_name}--{err_message}',
                             df_name,
                             column
                         )
-                        self.create_cell(code + "\n", "query", False)
+                        self.create_cell(code, "query", False)
                 return
-            elif command == "remove_dataframe":
+            elif command == "remove-dataframe":
                 df_name = data["df_name"]
                 self.remove_df_fun(df_name)
-                self.log_entry("remove_df", df_name)
-                # local store
                 del self.vis_spec[df_name]
                 self.remove_df_from_log(df_name)
                 return
             elif command == "add_current_selection":
                 value = json.loads(data["value"])
                 # parse it first!
+                s = data["value"]
+                debug_log(f"add_current_selection {s}")
                 self.handle_add_current_selection(value)
                 return
-            elif command == "log_entry":
-                try:
-                    self.log_entry(data["action"], data["metadata"])
-                except KeyError as err:
-                    self.send_debug_msg(f"Logging error {json.dumps(data)}")
             else:
                 m = f"Command {command} not handled!"
                 raise NotAllCaseHandledError(m)
         else:
             debug_log(f"Got message from JS Comm: {data}")
 
+
     def internal_current_selection(self, selections: List[SelectionValue], df_name):
         all_predicate = self.add_selection(selections)
         # now turn this into JSON
@@ -178,44 +169,28 @@ class UiComm(object):
             predicates = ", ".join(list(map(lambda v: v.to_str() if v.to_str() else "", all_predicate)))
             param_str = f"[{predicates}]"
         self.execute_selection(param_str, df_name)
-        self.log_entry("ui_selection", df_name)
 
 
     # note that this is extracted out for better debugging.
     def handle_add_current_selection(self, value: Dict):
         selections, df_name = self.get_predicate_info(value)
         self.internal_current_selection(selections, df_name)
-        
-
-    # def process_code(self, code: str):
-    #     assigned_dfs = self.get_dfs_from_code_str(code)
-    #     if len(assigned_dfs) == 0:
-    #         # do nothing
-    #         debug_log("no df to process")
-    #         return
-    #     # code_lines = []
-    #     for df in assigned_dfs:
-    #         if df.is_base_df:
-    #             # just execute it! since there is no configuration
-    #             df.show_profile()
-    #         else:
-    #             encoding = infer_encoding(df).__dict__
-    #             df.show(**encoding)
 
 
-    def set_comm(self, midas_instance_name: str):
+    def set_comm(self, midas_instance_name: str, logger_id: str):
         if self.is_in_ipynb:
             self.comm = Comm(target_name = MIDAS_CELL_COMM_NAME)
             self.comm.send({
-                "type": "midas_instance_name",
-                "value": midas_instance_name
+                "type": "initialize",
+                "name": midas_instance_name,
+                "loggerId": logger_id
             })
             self.comm.on_msg(self.handle_msg)
         else:
             self.comm = MockComm()
 
 
-    def register_recovery_comm(self, midas_instance_name: str):
+    def register_recovery_comm(self, midas_instance_name: str, logger_id: str):
         def target_func(comm, open_msg):
             # comm is the kernel Comm instance
             # msg is the comm_open message
@@ -228,7 +203,7 @@ class UiComm(object):
                     return
                 else:
                     debug_log("Received recovery request. Reopening comm...")
-                    self.set_comm(self.midas_instance_name)
+                    self.set_comm(self.midas_instance_name, logger_id)
                     debug_log("Clearing stored visualizations...")
                     self.vis_spec = {}
                     debug_log("Comm reopened. Rerunning logged commands...")
@@ -267,7 +242,7 @@ class UiComm(object):
 
         # if ISDEBUG: set_trace()
         records = dataframe_to_dict(df, FilterLabelOptions.unfiltered)
-        code = get_midas_code(df._ops)
+        code = get_midas_code(df._ops, self.midas_instance_name)
         # TODO: check if we even need to do the dumping
         data = json.dumps(records)
         hash_val = df._id + "_" + encoding.to_hash()
@@ -369,6 +344,7 @@ class UiComm(object):
             # "appendFlag": 1 if do_append else 0
         })
 
+
     def execute_selection(self, params: str, df_name: str):
         self.comm.send({
             "type": "execute_selection",
@@ -399,11 +375,12 @@ class UiComm(object):
             "columnName": column_name,
         })
 
-    def navigate_to_chart(self, df_name: DFName):
+    def log_start_task(self, task_id: str):
         self.comm.send({
-            "type": "navigate_to_vis",
-            "value": df_name
+            "type": "task-start",
+            "value": task_id
         })
+        print("Task logged", task_id)
 
     def get_predicate_info(self, selections: Dict) -> Tuple[List[SelectionValue], str]:
         """[summary]
@@ -433,39 +410,41 @@ class UiComm(object):
         vis = self.vis_spec[df_name]
         x = ColumnRef(vis.x, df_name)
         if not selection:
-            if vis.mark == "circle":
+            if vis.selection_dimensions == "xy":
                 y = ColumnRef(vis.y, df_name)
                 result.extend([
                     EmptySelection(x),
                     EmptySelection(y)
                 ])
-            else:
+            elif vis.selection_dimensions == "x":
+                result.extend([
+                    EmptySelection(x)
+                ])
+            elif vis.selection_dimensions == "y":
+                y = ColumnRef(vis.y, df_name)
                 result.extend([
                     EmptySelection(x)
                 ])
         else:
-            if vis.mark == "circle":
-                y = ColumnRef(vis.y, df_name)
-                # the predicate is either going to be x or y, or both
-                selections_added = 0
-                if vis.x in selection:
+            selections_added = 0
+            if vis.x in selection:
+                if vis.x_type == "quantitative" or vis.x_type == "temporal":
                     x_selection = NumericRangeSelection(x, selection[vis.x][0], selection[vis.x][1])
-                    result.append(x_selection)
-                    selections_added += 1
-                if vis.y in selection:
-                    y_selection = NumericRangeSelection(y, selection[vis.y][0], selection[vis.y][1])
-                    result.append(y_selection)
-                    selections_added += 1
-                if selections_added == 0:
-                    raise InternalLogicalError(f"Unknown selection {selection}");
-            elif vis.mark == "bar":
-                predicate = SetSelection(ColumnRef(vis.x, df_name), selection[vis.x])
-                result.append(predicate)
-            elif vis.mark == "line":
-                x_selection = NumericRangeSelection(ColumnRef(vis.x, df_name), selection[vis.x][0], selection[vis.x][1])
+                else:
+                    x_selection = SetSelection(ColumnRef(vis.x, df_name), selection[vis.x])
                 result.append(x_selection)
-            else:
-                raise InternalLogicalError(f"{vis.mark} not handled")
+                selections_added += 1
+            if vis.y in selection:
+                y = ColumnRef(vis.y, df_name)
+                
+                if vis.x_type == "quantitative" or vis.x_type == "temporal":
+                    y_selection = NumericRangeSelection(y, selection[vis.y][0], selection[vis.y][1])
+                else:
+                    y_selection = SetSelection(ColumnRef(vis.y, df_name), selection[vis.x])
+                result.append(y_selection)
+                selections_added += 1
+            if selections_added == 0:
+                raise InternalLogicalError(f"Unknown selection {selection}");
         return result, df_name
 
     # @logged(remove_on_chart_removal=False, "update_selection_shelf_selection_name")
@@ -488,9 +467,13 @@ class UiComm(object):
         new_name = sanitize_string_for_var_name(f"{col_name}_{df_name}_dist")
         if (is_string_dtype(col_value)):
             # we need to check the cardinarily
-            unique_vals = np.unique(col_value)
-            current_max_bins = len(unique_vals)
             code = get_basic_group_vis(new_name, df.df_name, col_name)
+            try:
+                unique_vals = np.unique(col_value)
+            except TypeError:
+                # with None value
+                return (code, False, f"Please handle None values from {col_name}!")
+            current_max_bins = len(unique_vals)
             if current_max_bins < MAX_BINS:
                 return (code, True, "")
             else:
@@ -507,7 +490,7 @@ class UiComm(object):
             unique_vals = np.unique(col_value[~np.isnan(col_value)])
             current_max_bins = len(unique_vals)
             if (current_max_bins < MAX_BINS):
-                code = f"{new_name} = {df.df_name}.group('{col_name}').vis()"
+                code = get_basic_group_vis(new_name, df.df_name, col_name)
                 return (code, True, "")
             else:
                 return get_numeric_distribution_code(current_max_bins, unique_vals, col_name, df.df_name, new_name, self.midas_instance_name)
